/*@file  	        cc1101.c
* @brief            This module contains the low level operations for CC1101.
* @details          
* @author           hyh
* @date             2022.6.6
* @version          1.0
* @copyright        Chengdu Ebyte Electronic Technology Co.Ltd
**********************************************************************************
*/
#include "cc1101.h"
/*SPI empty instruction*/
#define SPI_NOP 0xFF

/*
* @brief Corresponding to the output power(dBm)
*                    10,  7,   5,   0,   -5, -10, -15, -20.       0x12:-30dBm
*/
INT8U PaTable[8] = {0xC0,0xC8,0x84,0x60,0x68,0x34,0x1D,0x0E}; 
/*@brief Radio register Initialization:Set the params by the SmartRF Studio software*/
const CC1101_REG_DATA CC1101InitData[]=
{
  {CC1101_IOCFG0,     0x06},
  {CC1101_FIFOTHR,    0x47},
  {CC1101_PKTCTRL0,   0x01},//[2]CRC Disable
  {CC1101_CHANNR,     0x00},
  {CC1101_FSCTRL1,    0x06},
/************Frequency******************/ 
/*=433Mhz=*/
//  {CC1101_FREQ2,      0x10},
//  {CC1101_FREQ1,      0xA7},
//  {CC1101_FREQ0,      0x62},
/*=868Mhz=*/
  {CC1101_FREQ2,      0x21},
  {CC1101_FREQ1,      0x62},
  {CC1101_FREQ0,      0x76},
/*=915Mhz=*/
//  {CC1101_FREQ2,      0x23},
//  {CC1101_FREQ1,      0x31},
//  {CC1101_FREQ0,      0x3B}, 
/*************Data Rate*****************/  
/*=1.2Kbps,RX_BW:58Khz=*/  
  {CC1101_MDMCFG4,    0xF5},//[7:4]RX_BW
  {CC1101_MDMCFG3,    0x83},
  {CC1101_MDMCFG2,    0x13},//[2:0]30/32 sync word bits detected,[6:4]modulationformat:GFSK
/**********Channel Interval*************/
/*=333Khz=*/    
  {CC1101_MDMCFG1,     0x73},//[6:4]Preamble length 24
  {CC1101_MDMCFG0,     0xA4}, 
/**********Frequency deviation**********/
/*=14.3Khz=*/   
  {CC1101_DEVIATN,    0x31},
/***************************************/
  {CC1101_MCSM1,      0x3B},//0x3F:After receiving or transmitting, both enter RX mode
  {CC1101_MCSM0,      0x18},
  {CC1101_FOCCFG,     0x16},
  {CC1101_WORCTRL,    0xFB},
  {CC1101_FSCAL3,     0xE9},
  {CC1101_FSCAL2,     0x2A},
  {CC1101_FSCAL1,     0x00},
  {CC1101_FSCAL0,     0x1F},
  {CC1101_TEST2,      0x81},
  {CC1101_TEST1,      0x35}
};
/*@brief the array of the DataRate  By the SmartRF Studio*/
const INT16U DataRateTable[] =  //26MHz  //KBPS
{
    0x0583,     //0.6K
    0x0583,     //1.2K
    0x0643,     //2K
    0x06E4,     //3K
    0x0743,     //4K
    0X0793,     //5K
    0X07E4,     //6K
    0X081A,     //7K
    0X0843,     //8K
    0X086B,     //9K
    0x0893,     //10K
    0X08BC,     //11K
    0X08E4,     //12K
    0X0906,     //13K
    0X091A,     //14K
    0X092E,     //15K
    0X0943,     //16K
    0X0957,     //17K
    0X096B,     //18K
    0X097F,     //19K
    0x0993,     //20K
    0X09A7,     //21K
    0X09BC,     //22K
    0X09D0,     //23K
    0X09E4,     //24K
    0X09F8,     //25K
    0X0A06,     //26K
    0X0A10,     //27K
    0X0A1A,     //28K
    0X0A24,     //29K
    0x0A2E,     //30K
    0X0A39,     //31K
    0X0A43,     //32K
    0X0A4D,     //33K
    0X0A57,     //34K
    0X0A61,     //35K
    0X0A6B,     //36K
    0X0A75,     //37K
    0X0A7F,     //38K
    0X0A89,     //39K
    0x0A93,     //40K
};
/*!
 *  @brief       Write a byte to the specified register   
 *  @param       addr: the specified register address
 *               value: the value to be written
 *  @return      None
 *  @note        None
 *     
*/  
void CC1101_WriteReg(INT8U addr,INT8U value)
{
  CC_CSN_LOW( );
  SPI_ExchangeByte( addr | WRITE_SINGLE);
  SPI_ExchangeByte( value );
  CC_CSN_HIGH( );
}
/*!
 *  @brief       Write bytes to the specified register   
 *  @param       addr: the specified register address
 *               buffer: a buffer stores the values to be written
 *               size: how many bytes should be written
 *  @return      None
 *  @note        None
 *     
*/ 
void CC1101_WriteMultiReg(INT8U addr,INT8U *buffer,INT8U size)
{
  INT8U i;
  CC_CSN_LOW( );
  SPI_ExchangeByte( addr | WRITE_BURST );
  for(i=0;i<size;i++)
  {
    SPI_ExchangeByte(*(buffer+i));
  }
  CC_CSN_HIGH( );
}
/*!
 *  @brief       Write Command to CC1101 
 *  @param       Commamd Strobes
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_WriteCmd(INT8U command)
{
  CC_CSN_LOW( );
  SPI_ExchangeByte(command);
  CC_CSN_HIGH( );
}
/*!
 *  @brief       Read a byte from the specified register
 *  @param       addr: the address of the register
 *  @return      the byte read from the register
 *  @note        None
 *     
*/
INT8U CC1101_ReadReg(INT8U addr)
{
  INT8U read_data;
  CC_CSN_LOW( );
  SPI_ExchangeByte( addr | READ_SINGLE);
  read_data = SPI_ExchangeByte(SPI_NOP);
  CC_CSN_HIGH( );
  return read_data;
}
/*!
 *  @brief       Read bytes from the specified register continously
 *  @param       addr: the address of the register
 *               buffer: the buffer stores the data read from the register
 *               size: how many bytes should be read
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_ReadMultiReg(INT8U addr,INT8U *buffer,INT8U size)
{
  INT8U i;
  CC_CSN_LOW( );
  SPI_ExchangeByte( addr | READ_BURST);
  for(i=0;i<size;i++)
  {
    *(buffer+i) = SPI_ExchangeByte(SPI_NOP);
  }
  CC_CSN_HIGH( );
}
/*!
 *  @brief       Read Current status from the status register
 *  @param       addr: the address of the register
 *  @return      the value read from the register
 *  @note        None
 *     
*/
INT8U CC1101_ReadStatus(INT8U addr)
{
  INT8U status;
  CC_CSN_LOW( );
  SPI_ExchangeByte( addr | READ_BURST);
  status = SPI_ExchangeByte(SPI_NOP);
  CC_CSN_HIGH( );
  return status;
}
/*!
 *  @brief       Reset the CC1101
 *  @param       None
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_Reset(void)
{
  INT8U i;
  CC_CSN_HIGH();
  CC_CSN_LOW( );
  CC_CSN_HIGH( );
  for(i=0;i<100;i++);
  CC1101_WriteCmd(CC1101_SRES);
}
/*!
 *  @brief       Set the address and address mode of the CC1101
 *  @param       address  the device address 
 *               AddressModeaddress check mode
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetAddress(INT8U address,ADDR_MODE AddressMode)
{
  INT8U btmp = CC1101_ReadReg(CC1101_PKTCTRL1) & ~0x03;
  CC1101_WriteReg(CC1101_ADDR,address);
  if     ( AddressMode == BROAD_ALL )     {}
  else if( AddressMode == BROAD_NO  )     { btmp |= 0x01; }
  else if( AddressMode == BROAD_0   )     { btmp |= 0x02; }
  else if( AddressMode == BROAD_0AND255 ) { btmp |= 0x03; } 
  CC1101_WriteReg( CC1101_PKTCTRL1, btmp );
}
/*!
 *  @brief       Set the sync byte
 *  @param       sync: 16 bit sync 
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetSync(INT16U sync)
{
  CC1101_WriteReg(CC1101_SYNC1,(sync>>8)&0xFF);
  CC1101_WriteReg(CC1101_SYNC0,sync&0xFF);
}
/*!
 *  @brief       Set the datarate
 *  @param       datarate_index:  DataRateTables index
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetDataRate(INT8U datarate_index)
{
  INT8U mdmcfg4;
  if(datarate_index > 40){ return; }
  mdmcfg4 = CC1101_ReadReg(CC1101_MDMCFG4);
  mdmcfg4 &= 0xF0;
  mdmcfg4 |= ( 0x0F & ( DataRateTable[datarate_index] >> 8 ) );
  CC1101_WriteReg( CC1101_MDMCFG4, mdmcfg4 );
  CC1101_WriteReg( CC1101_MDMCFG3, DataRateTable[datarate_index] & 0xFF );
}
/*!
 *  @brief       Set the Channel
 *  @param       channel 
 *  @return      None
 *  @note        Beacause channel interval is 333KhzSelet one channel every 3 channels,
 *               Current Channel Interval:1Mhz!!!!!
*/
void CC1101_SetChannel(INT8U channel)
{
  INT8U channel_real;
  channel_real = channel * 3;
  CC1101_WriteReg( CC1101_CHANNR, channel_real);
}
/*!
 *  @brief       Set the output power
 *  @param       power_index: PaTable's index
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetPAPower(INT8U power_index)
{
  power_index &= 0x07;
  CC1101_WriteMultiReg(CC1101_PATABLE, PaTable + power_index, 1 );
}
/*!
 *  @brief       Initialize the CC1101
 *  @param        
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_Init(void)
{
  INT8U i;
  CC1101_Reset();
  for(i=0;i<sizeof(CC1101InitData)/sizeof(CC1101_REG_DATA);i++)
  {
    CC1101_WriteReg( CC1101InitData[i].addr, CC1101InitData[i].value );
  }
  CC1101_SetAddress(DEVICE_ADDRESS,BROAD_0AND255);
  CC1101_SetChannel(0);
  CC1101_WriteMultiReg(CC1101_PATABLE, PaTable, 8 );
  CC1101_SetPAPower( 0 );
  CC1101_WriteReg(CC1101_IOCFG0,0x46);//GDO0 interrupt
  CC1101_SetSync(0x8799);
  i = CC1101_ReadStatus( CC1101_PARTNUM );//Chip part number,for test, must be 0x80
  i = CC1101_ReadStatus( CC1101_VERSION );//Chip version number,for test, refer to the datasheet
}
/*!
 *  @brief       Set the device as Tx mode or Rx mode
 *  @param       mode selection 
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetTRMode(TRMODE mode)
{
  if(mode == TX_MODE)
  {
    CC1101_WriteCmd( CC1101_STX );
  }
  else if(mode == RX_MODE)
  {
    CC1101_WriteCmd( CC1101_SRX );
  }
}
/*!
 *  @brief       Set the CC1101 into the IDLE mode
 *  @param        
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SetIdle(void)
{
  CC1101_WriteCmd(CC1101_SIDLE);
}
/*!
 *  @brief       Flush the TX Buffer
 *  @param        
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_ClrTxBuffer(void)
{
  CC1101_SetIdle();//Must be Idle mode
  CC1101_WriteCmd( CC1101_SFTX );
}
/*!
 *  @brief       Flush the RX Buffer
 *  @param        
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_ClrRxBuffer(void)
{
  CC1101_SetIdle();//Must be Idle mode
  CC1101_WriteCmd( CC1101_SFRX );
}
/*!
 *  @brief       Get the recieve bytes count
 *  @param        
 *  @return      How many bytes have been received
 *  @note        None
 *     
*/
INT8U CC1101_GetRxCnt(void)
{
  return ( CC1101_ReadStatus( CC1101_RXBYTES )  & BYTES_IN_RXFIFO );
}
/*!
 *  @brief       Send data packet
 *  @param       txbuffer, The buffer stores data to be sent
 *               size, How many bytes should be sent
 *               mode, Broadcast or address check packet
 *  @return      None
 *  @note        None
 *     
*/
void CC1101_SendPacket( INT8U *txbuffer, INT8U size, TX_DATA_MODE mode )
{
  INT8U address;
  if(mode == BROADCAST) {address = 0;}
  else if(mode == ADDRESS_CHECK) {address = CC1101_ReadReg( CC1101_ADDR );}
  CC1101_ClrTxBuffer();
  if((CC1101_ReadReg(CC1101_PKTCTRL1) & ~0X03) != 0)
  {
    CC1101_WriteReg(CC1101_TXFIFO,size + 1);
    CC1101_WriteReg(CC1101_TXFIFO,address);
  }
  else
  {
    CC1101_WriteReg(CC1101_TXFIFO,size);
  }
  CC1101_WriteMultiReg(CC1101_TXFIFO,txbuffer,size);
  CC1101_SetTRMode(TX_MODE);
  while( CC_IRQ_READ()!= 0 );
  while( CC_IRQ_READ() == 0 );
  CC1101_ClrTxBuffer();
}

/*!
 *  @brief       Recieve data packet
 *  @param       rxBuffer, A buffer store the received data 
*  @return       1:recieve packet length  0: no data
 *  @note        None
 *     
*/
INT8U CC1101_RecievePacket(INT8U *rxbuffer)
{
  INT8U status[2];
  INT8U pktLen;
  volatile INT8U i=0;
  if ( CC1101_GetRxCnt() != 0 )
  {
      pktLen = CC1101_ReadReg(CC1101_RXFIFO);           // Read length byte
      if( ( CC1101_ReadReg( CC1101_PKTCTRL1 ) & ~0x03 ) != 0 )
      {
          i = CC1101_ReadReg(CC1101_RXFIFO);
      }
      if( pktLen == 0 )           { return 0; }
      else                        { pktLen --; }
      CC1101_ReadMultiReg(CC1101_RXFIFO, rxbuffer, pktLen); // Pull data
      CC1101_ReadMultiReg(CC1101_RXFIFO, status, 2);   // Read  status bytes
      CC1101_ClrRxBuffer();

      if( status[1] & CRC_OK ) {   return pktLen; }
      else                     {   return 0; }
  }
  else   {  return 0; }                               // Error
}
